import re
import csv

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse

from passerelle.base.models import BaseResource


class CsvDataSource(BaseResource):
    csv_file = models.FileField(_('CSV File'), upload_to='csv')
    columns_keynames = models.CharField(max_length=256,
                            verbose_name=_('Column keynames'),
                            help_text=_('ex: id,text,data1,data2'))
    skip_header = models.BooleanField(_('Skip first line'), default=False)

    category = _('Data Sources')

    class Meta:
        verbose_name = _('CSV File')

    @classmethod
    def get_verbose_name(cls):
        return cls._meta.verbose_name

    @classmethod
    def get_icon_class(cls):
        return 'grid'

    @classmethod
    def is_enabled(cls):
        return True

    @classmethod
    def get_add_url(cls):
        return reverse('csvdatasource-add')

    def get_absolute_url(self):
        return reverse('csvdatasource-detail', kwargs={'slug': self.slug})

    def get_data(self, filters=None, query=None, case_insensitive=False):
        if case_insensitive:
            if filters:
                for filter_key in filters.keys():
                    filters[filter_key] = filters[filter_key].lower()
            if query:
                query = query.lower()

        titles = [t.strip() for t in self.columns_keynames.split(',')]
        indexes = [titles.index(t) for t in titles if t]
        caption = [titles[i] for i in indexes]

        # validate query (a "text" column must exist)
        if query:
            if 'text' in titles:
                text_idx = titles.index('text')
            else:
                # no text column -> no query
                query = None

        # validate filters (appropriate columns must exist)
        if filters:
            for filter_key in filters.keys():
                if not filter_key in titles:
                    del filters[filter_key]

        def filter_row(row):
            if query:
                text = unicode(row[text_idx], 'utf-8')
                if case_insensitive:
                    text = text.lower()
                if not query in text:
                    return False
            if filters:
                for filter_key, filter_value in filters.items():
                    col = titles.index(filter_key)
                    text = unicode(row[col], 'utf-8')
                    if case_insensitive:
                        text = text.lower()
                    if filter_value != text:
                        return False
            return True

        content = self.csv_file.read()
        if not content:
            return None

        # remove BOM and detect CSV dialect
        content = content.decode('utf-8-sig').encode('utf-8')
        dialect = csv.Sniffer().sniff(content[:10240])

        reader = csv.reader(content.splitlines(), dialect)

        if self.skip_header:
            reader.next()

        data = []
        for row in reader:
            if (filters or query) and not filter_row(row):
                continue
            try:
                line = [row[i] for i in indexes]
                data.append(dict(zip(caption, line)))
            except IndexError:
                continue

        return data
